#ifndef HASHGRID_H
#define HASHGRID_H

#include "hashtable.h"
#include "vec3.h"

struct HashGrid3D
{
   float dx, overdx; // side-length of a grid cell and its reciprocal
   HashTable<Vec3i, int> grid; // hash table stores integer indices into array of particles

   explicit HashGrid3D(float dx_=1, int expected_size=512)
      : dx(dx_), overdx(1/dx_), grid(expected_size)
   {}

   void set_grid_size(float dx_)
   { dx=dx_; overdx=1/dx_; } // note: assumes grid is empty!

   void add_point(const Vec3f &x, int index)
   { grid.add(round(x*overdx), index); }

   void delete_point(const Vec3f &x, int index)
   { grid.delete_entry(round(x*overdx), index); }

   void add_box(const Vec3f &xmin, const Vec3f &xmax, int index);
   void delete_box(const Vec3f &xmin, const Vec3f &xmax, int index);
   
   unsigned int size(void) const
   { return grid.size(); }

   void clear(void)
   { grid.clear(); }

   void reserve(unsigned int expected_size)
   { grid.reserve(expected_size); }

   bool find_first_point(const Vec3f &x, int &index) const
   { return grid.get_entry(round(x*overdx), index); }

   bool find_point(const Vec3f &x, std::vector<int> &index_list) const;
   bool find_box(const Vec3f &xmin, const Vec3f &xmax, std::vector<int> &index_list) const;
};

#endif
