#include <cstdarg>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include "iomesh.h"

#define LINESIZE 1024 // maximum line size when reading .OBJ files

bool write_objfile(const TriMesh &mesh, const std::vector<Vec3f> &x, const std::vector<Vec3f> &normal,
                   const char *filename_format, ...)
{
   va_list ap;
   va_start(ap, filename_format);
   char *filename;
   vasprintf(&filename, filename_format, ap);
   std::ofstream output(filename, std::ofstream::binary);
   std::free(filename);
   va_end(ap);

   if(!output.good()) return false;

   for(unsigned int i=0; i<x.size(); ++i){
      output<<"v "<<x[i]<<std::endl;
      if(i<normal.size())
	 output<<"vn "<<normal[i]<<std::endl;
   }
   for(unsigned int t=0; t<mesh.tri.size(); ++t)
      output<<"f "<<mesh.tri[t][0]+1<<' '<<mesh.tri[t][1]+1<<' '<<mesh.tri[t][2]+1<<std::endl; // correct for 1-based indexing in OBJ files
   return output.good();
}

